require "active_hash_ext/version"

module ActiveHashExt
  def self.included base
    base.extend ClassMethods
  end

  module ClassMethods

    def read_schema(table, path)
      tables = File.read(path).split(/^\s*create_/)[1..-1].map {|table_data| Table.parse table_data }
      table = tables.select do |_table|
        _table.name == table
      end
      unless table.first.nil?
        table.first.attributes.map do |attribute|
          attribute.name.to_sym
        end
      end

      if table.first.nil?
        raise "Table Name not Found!"
      end

    end

    def destroy_all
      delete_all
    end
  end

  def update(attributes)
    self.attributes.merge(attributes).each do |attribute|
      send("#{attribute.first}=",attribute.last)
    end
  end

  def update_attributes(attributes)
    update(attributes)
  end



end



  require 'active_support/inflector'

  class Table

    attr_reader :attributes, :name

    def initialize(name, attributes)
      @name, @attributes = name, attributes
    end

    def to_script(target)
      return "rails generate #{target} #{modelize name} #{attributes.map(&:to_script).reject{|x| x.nil? || x.empty?}.join(' ')}" if target == "scaffold"
      return "rails generate #{target} #{modelize name} #{attributes.map(&:to_script).reject{|x| x.nil? || x.empty?}.join(' ')}" if target == "factory_girl:model"
    end

    def self.parse(table_data)
      return unless name = table_data[/table "(.+)"/]
      name = $1
      atts = table_data.lines.to_a.select {|line| line =~ /t\.\w+/ }.map {|att| Attribute.parse att }
      Table.new(name, atts)
    end

    private
    def modelize (string)
      string.camelize.singularize
    end

  end


class Attribute

  attr_reader :name, :type

  def initialize(type, name)
    @name, @type = name, type
  end

  def to_script
    "#{name}:#{type}" unless ["created_at","updated_at"].include? name
  end

  def self.parse(attribute)
    match = attribute.match(/t\.(\w+)\s+"(\w+)"/)
    if match
      Attribute.new(*match.captures)
    end
  end


end

